@@ -1,7 +1,7 @@
-/* gdbmerrno.c - convert gdbm errors into english. */
+/* gdbmerrno.c - convert gdbm errors into English. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1993-2021 Free Software Foundation, Inc.
+   Copyright (C) 1993-2022 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -117,9 +117,9 @@ const char * const gdbm_errlist[_GDBM_MAX_ERRNO+1] = {
   [GDBM_ITEM_NOT_FOUND]         = N_("Item not found"),
   [GDBM_REORGANIZE_FAILED]      = N_("Reorganize failed"),
   [GDBM_CANNOT_REPLACE]         = N_("Cannot replace"),
-  [GDBM_ILLEGAL_DATA]           = N_("Illegal data"),
+  [GDBM_MALFORMED_DATA]         = N_("Malformed data"),
   [GDBM_OPT_ALREADY_SET]        = N_("Option already set"),
-  [GDBM_OPT_ILLEGAL]            = N_("Illegal option"),
+  [GDBM_OPT_BADVAL]             = N_("Bad option value"),
   [GDBM_BYTE_SWAPPED]           = N_("Byte-swapped file"),
   [GDBM_BAD_FILE_OFFSET]        = N_("File header assumes wrong off_t size"),
   [GDBM_BAD_OPEN_FLAGS]         = N_("Bad file flags"),
