@@ -1,7 +1,7 @@
 /* gdbmsetopt.c - set options pertaining to a GDBM descriptor. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1993-2021 Free Software Foundation, Inc.
+   Copyright (C) 1993-2022 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ setopt_gdbm_setcachesize (GDBM_FILE dbf, void *optval, int optlen)
   /* Optval will point to the new size of the cache. */
   if (get_size (optval, optlen, &sz))
     {     
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }  
   return _gdbm_cache_init (dbf, sz);
@@ -67,13 +67,39 @@ setopt_gdbm_getcachesize (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (size_t))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   *(size_t*) optval = dbf->cache_size;
   return 0;
 }
 
+static int
+setopt_gdbm_setcacheauto (GDBM_FILE dbf, void *optval, int optlen)
+{
+  int n;
+
+  if ((n = getbool (optval, optlen)) == -1)
+    {     
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
+      return -1;
+    }
+  dbf->cache_auto = n;
+  return 0;
+}
+
+static int
+setopt_gdbm_getcacheauto (GDBM_FILE dbf, void *optval, int optlen)
+{
+  if (!optval || optlen != sizeof (int))
+    {
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
+      return -1;
+    }
+  *(int*) optval = !!dbf->cache_auto;
+  return 0;
+}
+
 /* Obsolete form of GDBM_SETSYNCMODE. */
 static int
 setopt_gdbm_fastmode (GDBM_FILE dbf, void *optval, int optlen)
@@ -82,7 +108,7 @@ setopt_gdbm_fastmode (GDBM_FILE dbf, void *optval, int optlen)
 
   if ((n = getbool (optval, optlen)) == -1)
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   dbf->fast_write = n;
@@ -97,7 +123,7 @@ setopt_gdbm_setsyncmode (GDBM_FILE dbf, void *optval, int optlen)
   /* Optval will point to either true or false. */
   if ((n = getbool (optval, optlen)) == -1)
     { 
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   dbf->fast_write = !n;
@@ -109,7 +135,7 @@ setopt_gdbm_getsyncmode (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (int))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   *(int*) optval = !dbf->fast_write;
@@ -125,7 +151,7 @@ setopt_gdbm_setcentfree (GDBM_FILE dbf, void *optval, int optlen)
   /* Optval will point to either true or false. */
   if ((n = getbool (optval, optlen)) == -1)
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   dbf->central_free = n;
@@ -137,7 +163,7 @@ setopt_gdbm_getcentfree (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (int))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   *(int*) optval = !dbf->central_free;
@@ -153,7 +179,7 @@ setopt_gdbm_setcoalesceblks (GDBM_FILE dbf, void *optval, int optlen)
   /* Optval will point to either true or false. */
   if ((n = getbool (optval, optlen)) == -1)
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   dbf->coalesce_blocks = n;
@@ -165,7 +191,7 @@ setopt_gdbm_getcoalesceblks (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (int))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   *(int*) optval = dbf->coalesce_blocks;
@@ -180,7 +206,7 @@ setopt_gdbm_setmmap (GDBM_FILE dbf, void *optval, int optlen)
   
   if ((n = getbool (optval, optlen)) == -1)
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   gdbm_file_sync (dbf);
@@ -206,7 +232,7 @@ setopt_gdbm_getmmap (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (int))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   *(int*) optval = dbf->memory_mapping;
@@ -222,7 +248,7 @@ setopt_gdbm_setmaxmapsize (GDBM_FILE dbf, void *optval, int optlen)
 
   if (get_size (optval, optlen, &sz))
     { 
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   dbf->mapped_size_max = ((sz + page_size - 1) / page_size) * page_size;
@@ -235,7 +261,7 @@ setopt_gdbm_getmaxmapsize (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (size_t))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   *(size_t*) optval = dbf->mapped_size_max;
@@ -248,20 +274,30 @@ setopt_gdbm_getflags (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (int))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   else
     {
       int flags = dbf->read_write;
+
       if (!dbf->fast_write)
 	flags |= GDBM_SYNC;
+      
       if (!dbf->file_locking)
 	flags |= GDBM_NOLOCK;
+
       if (!dbf->memory_mapping)
 	flags |= GDBM_NOMMAP;
       else if (dbf->mmap_preread)
 	flags |= GDBM_PREREAD;
+
+      if (dbf->cloexec)
+	flags |= GDBM_CLOEXEC;
+      
+      if (dbf->header->header_magic == GDBM_NUMSYNC_MAGIC)
+	flags |= GDBM_NUMSYNC;
+      
       *(int*) optval = flags;
     }
   return 0;
@@ -272,7 +308,7 @@ setopt_gdbm_getdbname (GDBM_FILE dbf, void *optval, int optlen)
 {
   if (!optval || optlen != sizeof (char*))
     {
-      GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+      GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
       return -1;
     }
   else
@@ -297,7 +333,55 @@ setopt_gdbm_getblocksize (GDBM_FILE dbf, void *optval, int optlen)
       return 0;
     }
   
-  GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+  GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
+  return -1;
+}
+
+static int
+setopt_gdbm_getdbformat (GDBM_FILE dbf, void *optval, int optlen)
+{
+  if (optval && optlen == sizeof (int))
+    {
+      switch (dbf->header->header_magic)
+	{
+	case GDBM_OMAGIC:
+	case GDBM_MAGIC:
+	  *(int*)optval = 0;
+	  break;
+      
+	case GDBM_NUMSYNC_MAGIC:
+	  *(int*)optval = GDBM_NUMSYNC;
+	}
+      return 0;
+    }
+  
+  GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
+  return -1;
+}
+
+static int
+setopt_gdbm_getdirdepth (GDBM_FILE dbf, void *optval, int optlen)
+{
+  if (optval && optlen == sizeof (int))
+    {
+      *(int*) optval = dbf->header->dir_bits;
+      return 0;
+    }
+  
+  GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
+  return -1;
+}
+
+static int
+setopt_gdbm_getbucketsize (GDBM_FILE dbf, void *optval, int optlen)
+{
+  if (optval && optlen == sizeof (int))
+    {
+      *(int*) optval = dbf->header->bucket_elems;
+      return 0;
+    }
+  
+  GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
   return -1;
 }
 
@@ -322,6 +406,11 @@ static setopt_handler setopt_handler_tab[] = {
   [GDBM_GETFLAGS]        = setopt_gdbm_getflags,
   [GDBM_GETDBNAME]       = setopt_gdbm_getdbname,
   [GDBM_GETBLOCKSIZE]    = setopt_gdbm_getblocksize,
+  [GDBM_GETDBFORMAT]     = setopt_gdbm_getdbformat,
+  [GDBM_GETDIRDEPTH]     = setopt_gdbm_getdirdepth,
+  [GDBM_GETBUCKETSIZE]   = setopt_gdbm_getbucketsize,
+  [GDBM_GETCACHEAUTO]    = setopt_gdbm_getcacheauto,
+  [GDBM_SETCACHEAUTO]    = setopt_gdbm_setcacheauto,
 };
   
 int
@@ -335,6 +424,6 @@ gdbm_setopt (GDBM_FILE dbf, int optflag, void *optval, int optlen)
       && setopt_handler_tab[optflag])
     return setopt_handler_tab[optflag] (dbf, optval, optlen);
   
-  GDBM_SET_ERRNO (dbf, GDBM_OPT_ILLEGAL, FALSE);
+  GDBM_SET_ERRNO (dbf, GDBM_OPT_BADVAL, FALSE);
   return -1;
 }
